@@ -1,24 +1,18 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2014 Jan Varho
+# Copyright (c) 2014, Jan Varho
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
+# Permission to use, copy, modify, and/or distribute this software for any
+# purpose with or without fee is hereby granted, provided that the above
+# copyright notice and this permission notice appear in all copies.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 """Scrypt implementation that calls into system libsodium"""
 
@@ -40,24 +34,51 @@ else:
     from . import pylibsodium_salsa as scr_mod
 
 
-_libsodium_soname = ctypes.util.find_library('sodium')
-if _libsodium_soname is None:
+_lib_soname = ctypes.util.find_library('sodium')
+if _lib_soname is None:
     raise ImportError('Unable to find libsodium')
+try:
+    _lib = ctypes.CDLL(_lib_soname)
+except OSError:
+    raise ImportError('Unable to load libsodium: ' + _lib_soname)
 
 try:
-    _libsodium = ctypes.CDLL(_libsodium_soname)
-    _scrypt = _libsodium.crypto_pwhash_scryptxsalsa208sha256
-    _scrypt_str = _libsodium.crypto_pwhash_scryptxsalsa208sha256_str
-    _scrypt_str_chk = _libsodium.crypto_pwhash_scryptxsalsa208sha256_str_verify
-    _scrypt_str_bytes = _libsodium.crypto_pwhash_scryptxsalsa208sha256_strbytes
-    _scrypt_salt = _libsodium.crypto_pwhash_scryptxsalsa208sha256_saltbytes
-    _scrypt_salt = _scrypt_salt()
-    if _scrypt_str_bytes() != 102:
-        raise ImportError('Incompatible libsodium: ' + _libsodium_soname)
-except OSError:
-    raise ImportError('Unable to load libsodium: ' + _libsodium_soname)
+    _scrypt_ll = _lib.crypto_pwhash_scryptsalsa208sha256_ll
+    _scrypt_ll.argtypes = [
+        c_void_p,  # passwd
+        c_size_t,  # passwdlen
+        c_void_p,  # salt
+        c_size_t,  # saltlen
+        c_uint64,  # N
+        c_uint32,  # r
+        c_uint32,  # p
+        c_void_p,  # buf
+        c_size_t,  # buflen
+    ]
 except AttributeError:
-    raise ImportError('Incompatible libsodium: ' + _libsodium_soname)
+    _scrypt_ll = None
+
+try:
+    _scrypt = _lib.crypto_pwhash_scryptsalsa208sha256
+    _scrypt_str = _lib.crypto_pwhash_scryptsalsa208sha256_str
+    _scrypt_str_chk = _lib.crypto_pwhash_scryptsalsa208sha256_str_verify
+    _scrypt_str_bytes = _lib.crypto_pwhash_scryptsalsa208sha256_strbytes()
+    _scrypt_salt = _lib.crypto_pwhash_scryptsalsa208sha256_saltbytes()
+    if _scrypt_str_bytes != 102 and not _scrypt_ll:
+        raise ImportError('Incompatible libsodium: ' + _lib_soname)
+except AttributeError:
+    try:
+        _scrypt = _lib.crypto_pwhash_scryptxsalsa208sha256
+        _scrypt_str = _lib.crypto_pwhash_scryptxsalsa208sha256_str
+        _scrypt_str_chk = _lib.crypto_pwhash_scryptxsalsa208sha256_str_verify
+        _scrypt_str_bytes = _lib.crypto_pwhash_scryptxsalsa208sha256_strbytes()
+        _scrypt_salt = _lib.crypto_pwhash_scryptxsalsa208sha256_saltbytes
+        _scrypt_salt = _scrypt_salt()
+        if _scrypt_str_bytes != 102 and not _scrypt_ll:
+            raise ImportError('Incompatible libsodium: ' + _lib_soname)
+    except AttributeError:
+        if not _scrypt_ll:
+            raise ImportError('Incompatible libsodium: ' + _lib_soname)
 
 _scrypt.argtypes = [
     c_void_p,  # out
@@ -85,7 +106,7 @@ _scrypt_str_chk.argtypes = [
 
 
 def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
-    """Derives a 64-byte hash using the scrypt key-derivarion function
+    """Returns a key derived using the scrypt key-derivarion function
 
     N must be a power of two larger than 1 but no larger than 2 ** 63 (insane)
     r and p must be positive numbers such that r * p < 2 ** 30
@@ -105,15 +126,18 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
     """
     check_args(password, salt, N, r, p, olen)
 
+    if _scrypt_ll:
+        out = ctypes.create_string_buffer(olen)
+        if _scrypt_ll(password, len(password), salt, len(salt),
+                      N, r, p, out, olen):
+            raise ValueError
+        return out.raw
+
     if len(salt) != _scrypt_salt or r != 8 or (p & (p - 1)) or (N*p <= 512):
         return scr_mod.scrypt(password, salt, N, r, p, olen)
 
-    for s in range(1, 64):
-        if 2**s == N:
-            break
-    for t in range(0, 30):
-        if 2**t == p:
-            break
+    s = next(i for i in range(1, 64) if 2**i == N)
+    t = next(i for i in range(0, 30) if 2**i == p)
     m = 2**(10 + s)
     o = 2**(5 + t + s)
     if s > 53 or t + s > 58:
@@ -144,15 +168,12 @@ def scrypt_mcf(password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p,
 
     if (salt is not None or r != 8 or (p & (p - 1)) or (N*p <= 512) or
         prefix not in (SCRYPT_MCF_PREFIX_7, SCRYPT_MCF_PREFIX_s1,
-                       SCRYPT_MCF_PREFIX_ANY)):
+                       SCRYPT_MCF_PREFIX_ANY) or
+        _scrypt_ll):
         return mcf_mod.scrypt_mcf(scrypt, password, salt, N, r, p, prefix)
 
-    for s in range(1, 32):
-        if 2**s == N:
-            break
-    for t in range(0, 8):
-        if 2**t == p:
-            break
+    s = next(i for i in range(1, 32) if 2**i == N)
+    t = next(i for i in range(0, 8) if 2**i == p)
     m = 2**(10 + s)
     o = 2**(5 + t + s)
     mcf = ctypes.create_string_buffer(102)
@@ -169,7 +190,7 @@ def scrypt_mcf(password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p,
 
 def scrypt_mcf_check(mcf, password):
     """Returns True if the password matches the given MCF hash"""
-    if mcf_mod._scrypt_mcf_7_is_standard(mcf):
+    if mcf_mod._scrypt_mcf_7_is_standard(mcf) and not _scrypt_ll:
         return _scrypt_str_chk(mcf, password, len(password)) == 0
     return mcf_mod.scrypt_mcf_check(scrypt, mcf, password)
 
