@@ -1,24 +1,18 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2014 Jan Varho
+# Copyright (c) 2014, Jan Varho
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
+# Permission to use, copy, modify, and/or distribute this software for any
+# purpose with or without fee is hereby granted, provided that the above
+# copyright notice and this permission notice appear in all copies.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 """Tests scrypt and PBKDF2 implementations"""
 
@@ -30,9 +24,18 @@ import unittest
 
 class ScryptTests(unittest.TestCase):
     """Tests an scrypt implementation from module"""
+    set_up_lambda = None
+    tear_down_lambda = None
+
     def setUp(self):
         if not self.module:
             self.skipTest('module not tested')
+        if self.set_up_lambda:
+            self.set_up_lambda()
+
+    def tearDown(self):
+        if self.tear_down_lambda:
+            self.tear_down_lambda()
 
     def _test_vector(self, vector):
         pw, s, N, r, p, h, m = vector
@@ -118,6 +121,17 @@ class ScryptTests(unittest.TestCase):
             None
         ))
 
+    def test_vector7(self):
+        if self.fast:
+            self.skipTest('slow testcase')
+        self._test_vector((
+            b'pleaseletmein', b'X'*32, 2**10, 8, 2,
+            b'1693cc02b680b18b3d0a874d459d23a5f63ff4f9de0fb5917ef899226af6'
+            b'bd33d3a3dfe569d3b6f4f762f0cb64f5f406d485aca501a54645d7389fe6'
+            b'e28e261e',
+            None
+        ))
+
     def test_bytes_enforced(self):
         self.assertRaises(TypeError, self.module.scrypt, u'pass', b'salt')
         self.assertRaises(TypeError, self.module.scrypt, 42, b'salt')
@@ -250,10 +264,10 @@ class ScryptTests(unittest.TestCase):
         if self.fast:
             self.skipTest('slow testcase')
         p = b'pleaseletmein'
-        m1 = self.module.scrypt_mcf(p, None, 2**10, 8, 1, b'$7$')
+        m1 = self.module.scrypt_mcf(p, None, 2**10, 8, 2, b'$7$')
         self.assertTrue(m1.startswith(b'$7$'))
         self.assertTrue(self.module.scrypt_mcf_check(m1, p))
-        m2 = self.module.scrypt_mcf(p, None, 2**10, 8, 1, b'$s1$')
+        m2 = self.module.scrypt_mcf(p, None, 2**10, 8, 2, b'$s1$')
         self.assertTrue(m2.startswith(b'$s1$'))
         self.assertTrue(self.module.scrypt_mcf_check(m1, p))
 
@@ -263,7 +277,7 @@ class ScryptTests(unittest.TestCase):
             b'$ENlyo6fGw4PCcDBOFepfSZjFUnVatHzCcW55.ZGz3B0'
         )
         self.assertTrue(self.module.scrypt_mcf_check(m1, p))
-        m2 = self.module.scrypt_mcf(p, None, 4, 8, 1, b'$7$')
+        m2 = self.module.scrypt_mcf(p, b'NaCl', 4, 8, 1, b'$7$')
         self.assertTrue(self.module.scrypt_mcf_check(m2, p))
 
     def test_mcf_unknown(self):
@@ -353,6 +367,27 @@ if __name__ == "__main__":
         from . import pylibsodium
         suite.addTest(load_scrypt_suite('pylibsodiumTests',
                                         pylibsodium, True))
+        from . import pylibscrypt
+        loader = unittest.defaultTestLoader
+        def set_up_ll(self):
+            if not self.module._scrypt_ll:
+                self.skipTest('no ll')
+            self.tmp_ll = self.module._scrypt_ll
+            self.tmp_scr = self.module.scr_mod
+            self.module._scrypt_ll = None
+            self.module.scr_mod = pylibscrypt
+        def tear_down_ll(self):
+            self.module._scrypt_ll = self.tmp_ll
+            self.module.scr_mod = self.tmp_scr
+        tmp = type(
+            'pylibsodiumFallbackTests', (ScryptTests,),
+            {
+                'module': pylibsodium, 'fast': False,
+                'set_up_lambda': set_up_ll,
+                'tear_down_lambda': tear_down_ll,
+            }
+        )
+        suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(tmp))
     except ImportError:
         raise
         suite.addTest(load_scrypt_suite('pylibsodiumTests', None, True))
